pub enum PrivilegeMode {
Unprivileged,
Privileged,
Auto,
}Expand description
Determines how the sandbox operates with respect to privileges
Variants
Unprivileged
Use only unprivileged mechanisms: user namespaces + seccomp + landlock + setrlimit. Does NOT require root. Fails if essential unprivileged features are unavailable.
Privileged
Use all available mechanisms including privileged ones: all namespaces + cgroups + chroot + seccomp. Requires root. Fails if not running as root.
Auto
Automatically detect the best available mode. Uses privileged mode if running as root, otherwise falls back to unprivileged.
Implementations
Sourceimpl PrivilegeMode
impl PrivilegeMode
Sourcepub fn resolve(&self, caps: &SystemCapabilities) -> ResolvedMode
pub fn resolve(&self, caps: &SystemCapabilities) -> ResolvedMode
Resolve Auto mode to a concrete mode based on system capabilities